﻿//-----------------------------------------------------------------------------
// <copyright file="NdfdClient.cs" company="Eclectrics, LLC">
//     © 2012 Eclectrics, LLC. All rights reserved.
// </copyright>
// Free for all uses under the Creative Commons Attribution 3.0 United States License
// http://creativecommons.org/licenses/by/3.0/us/
//-----------------------------------------------------------------------------

namespace Eclectrics.NationalWeatherServiceLibrary
{
    using System;
    using System.Diagnostics;
    using System.Drawing;
    using System.ServiceModel;
    using System.ServiceModel.Channels;
    using System.Text;
    using System.Xml;

    using Eclectrics.NationalWeatherServiceLibrary.NdfdServiceReference;
    using Eclectrics.SimpleTextMessageEncoderLibrary;

    /// <summary>
    /// A NDFD web service wrapper.
    /// </summary>
    public class NdfdClient
    {
        /// <summary>
        /// The web service address.
        /// </summary>
        private const string EndpointAddress = "http://graphical.weather.gov/xml/SOAP_server/ndfdXMLserver.php";

        /// <summary>
        /// The web service proxy.
        /// </summary>
        private ndfdXMLPortType client;

        /// <summary>
        /// The factory that will create the proxy.
        /// </summary>
        private ChannelFactory<ndfdXMLPortType> channelFactory;

        /// <summary>
        /// Open the channel.
        /// </summary>
        /// <param name="explaination">The explaination of success or failure.</param>
        /// <returns>True if successful, otherwise false.</returns>
        public bool Initialize(out string explaination)
        {
            bool success;

            try
            {
                var binding = new CustomBinding();

                var encoder = new SimpleTextMessageEncoderBindingElement
                    { Encoding = "ISO-8859-1", MessageVersion = MessageVersion.Soap11, MediaType = "text/xml" };
                binding.Elements.Add(encoder);

                var transport = new HttpTransportBindingElement
                    { AllowCookies = false, HostNameComparisonMode = HostNameComparisonMode.StrongWildcard };
                binding.Elements.Add(transport);

                this.channelFactory = new ChannelFactory<ndfdXMLPortType>(binding, new EndpointAddress(EndpointAddress));
                this.channelFactory.Open();

                this.client = this.channelFactory.CreateChannel();

                explaination = "Success";
                success = true;
            }
            catch (Exception ex)
            {
                explaination = ex.Message;
                success = false;
            }

            return success;
        }

        /// <summary>
        /// Close the channel.
        /// </summary>
        public void Cleanup()
        {
            if (this.channelFactory != null && this.channelFactory.State == CommunicationState.Opened)
            {
                this.channelFactory.Close();
            }
        }

        /// <summary>
        /// Get the longitude/latitude for a US Zip code using the NDFD Web service.
        /// </summary>
        /// <param name="zip">The requested zip code.</param>
        /// <param name="lonlatPoint">The location corresponding to the requested zip code.</param>
        /// <param name="explaination">An explaination of success or failure.</param>
        /// <returns>True if successful, otherwise false.</returns>
        public bool GetLonLatForZip(string zip, out PointF lonlatPoint, out string explaination)
        {
            bool success = false;
            lonlatPoint = Point.Empty;  // (0.0, 0.0) is not a valid point in the US

            if (this.client == null && !this.Initialize(out explaination))
            {
                return false;
            }

            var doc = new XmlDocument();

            try
            {
                if (this.client != null)
                {
                    doc.LoadXml(this.client.LatLonListZipCode(zip));
                }
                else
                {
                    explaination = "Failed to create client.";
                    return false;
                }
            }
            catch (Exception ex)
            {
                explaination = "Request to NDFD service failed to get Long/Lat for Zip: " + ex.Message;
                return false;
            }

            var root = doc.DocumentElement;
            if (root == null)
            {
                explaination = "Request to NDFD service failed to return valid XML document";
                return false;
            }

            var latLonNode = root.SelectSingleNode("latLonList");

            if (latLonNode != null)
            {
                var separators = new[] { ',' };
                var lonLatStrArray = latLonNode.InnerText.Split(separators, StringSplitOptions.None);

                if (lonLatStrArray.Length == 2)
                {
                    float latitude;
                    if (float.TryParse(lonLatStrArray[0], out latitude))
                    {
                        float longitude;
                        success = float.TryParse(lonLatStrArray[1], out longitude);

                        if (success)
                        {
                            lonlatPoint = new PointF(longitude, latitude);
                            explaination = "Success";
                        }
                        else
                        {
                            explaination = "Could not parse returned latitude string.";
                        }
                    }
                    else
                    {
                        explaination = "Could not parse returned longitude string.";
                    }
                }
                else
                {
                    explaination = "Parsed XML did not contain one longitude and one latitude string.";
                }
            }
            else
            {
                explaination = "Returned XML did not contain a latLonList.";
            }

            return success;
        }

        /// <summary>
        /// Get a weather forecast for the requested location and number of days from the NWS NDFD.
        /// </summary>
        /// <param name="longitude">longitude for forecast</param>
        /// <param name="latitude">latitude for forecast</param>
        /// <param name="days">number of days in forecast</param>
        /// <returns>True if successful, otherwise false.</returns>
        public string GetXmlForecastFromLatLon(decimal longitude, decimal latitude, string days)
        {
            string result;

            try
            {
                result = this.client.NDFDgenByDay(
                    latitude,
                    longitude,
                    DateTime.Now,
                    days,
                    unitType.e, 
                    formatType.Item12hourly);

                Debug.WriteLine(result);
            }
            catch (CommunicationException comEx)
            {
                var text = new StringBuilder();
                text.AppendLine("Web Service Error: ");
                text.AppendLine(comEx.InnerException.Message);
                result = text.ToString();
            }
            catch (Exception ex)
            {
                var text = new StringBuilder();
                text.AppendLine("Error: ");
                text.AppendLine(ex.Message);
                result = text.ToString();
            }

            return result;
        }
    }
}
