﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Eclectrics.NationalWeatherServiceLibrary;

namespace Tester
{
	public partial class TesterForm : Form
	{
		private enum AppState
		{
			NeedZip,
			GettingLocation,
			ReadyForGetForcast,
			GettingForecast,
			HaveForecast
		} ;

		private NdfdClient _client;
		private const int StdZipLength = 5;
		private AppState _appState;


		public TesterForm()
		{
			InitializeComponent();
		}


		protected override void OnCreateControl()
		{
			base.OnCreateControl();

			_client = new NdfdClient();

			SetState(AppState.NeedZip);

			_locationBkgndWorker.DoWork += delegate(object sender, DoWorkEventArgs args)
           	{
           		args.Result = GetLonLatFromZip(args.Argument as string);
           	};

			_locationBkgndWorker.RunWorkerCompleted += delegate(object sender, RunWorkerCompletedEventArgs args)
           	{
				if (args.Error != null)
				{
					MessageBox.Show(this, args.Error.Message);
				}
				else if (args.Cancelled)
				{
					MessageBox.Show(this, "Get Location cancelled.");
				}
				else
				{
					GetLonLatResult result = (GetLonLatResult)args.Result;
					if (result.Success)
					{
						_longitudeTextBox.Text = result.LonLat.X.ToString();
						_latitudeTextBox.Text = result.LonLat.Y.ToString();
						SetState(AppState.ReadyForGetForcast);
					}	
					else
					{
						MessageBox.Show(this, "The location for the supplied ZIP could not be found.", "Bad ZIP");
						SetState(AppState.NeedZip);						
					}
				}
           	};

			_forecastBkgndWorker.DoWork += delegate(object sender, DoWorkEventArgs args)
           	{
           		args.Result = GetXmlForecastFromLonLat(args.Argument as ForecastSetupData);
           	};

			_forecastBkgndWorker.RunWorkerCompleted += delegate(object sender, RunWorkerCompletedEventArgs args)
           	{
				if (args.Error != null)
				{
					MessageBox.Show(this, args.Error.Message);
				}
				else if (args.Cancelled)
				{
					MessageBox.Show(this, "Get Forecast cancelled.");
				}
				else
				{
					GetXmlForecastResult result = (GetXmlForecastResult) args.Result;
					if (result.Success)
					{
						_xmlTree.LoadXml(result.XmlForecast);
						SetState(AppState.HaveForecast);
					}
					else
					{
						SetState(AppState.ReadyForGetForcast);
					}
				}
           	};
		}


		/// <summary>
		/// Evaluate contents of Zip text box - if it seems valid, send a request
		/// to the NDFD for the corresponding longitude and latitude.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void _zipTextBox_TextChanged(object sender, EventArgs e)
		{
			_longitudeTextBox.Text = "";
			_latitudeTextBox.Text = "";
			_xmlTree.Clear();

			if (IsZipTextValid)
			{
				SetState(AppState.GettingLocation);
				_locationBkgndWorker.RunWorkerAsync(_zipTextBox.Text);
			}
		}


		/// <summary>
		/// If we have a longitude and latitude, send a request to the NDFD for a
		/// weather forecast.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void _forecastButton_Click(object sender, EventArgs e)
		{
			decimal longitude, latitude;

			if (_client != null && 
				decimal.TryParse(_longitudeTextBox.Text, out longitude) && 
				decimal.TryParse(_latitudeTextBox.Text, out latitude))
			{
				SetState(AppState.GettingForecast);
				_forecastBkgndWorker.RunWorkerAsync(new ForecastSetupData(longitude, latitude, _daysNumericUpDown.Text));				
			}
		}


		/// <summary>
		/// Apply nominal rules to the contents of _zipTextBox to see if it could be a zip code.
		/// </summary>
		private bool IsZipTextValid
		{
			get
			{
				uint zipCode;
				// If the length is right, and it parses into a UINT it could be a valid zip code.
				return (_zipTextBox.Text.Length == StdZipLength && uint.TryParse(_zipTextBox.Text, out zipCode));
			}
		}


		private GetLonLatResult GetLonLatFromZip(string zip)
		{
			GetLonLatResult result = new GetLonLatResult();

			if (_client != null)
			{
				string explaination;
				PointF lonLat;
				result.Success = _client.GetLonLatForZip(zip, out lonLat, out explaination);
				result.LonLat = lonLat;
				result.Explaination = explaination;
			}
			else
			{
				result.Success = false;
				result.LonLat = PointF.Empty;
				result.Explaination = "Failed to get longitude/latitude - NDFD client was not initialized.";
			}
			return result;
		}


		private GetXmlForecastResult GetXmlForecastFromLonLat(ForecastSetupData setupData)
		{
			GetXmlForecastResult result = new GetXmlForecastResult();

			if (_client != null)
			{
				try
				{
					result.Success = true;
					result.XmlForecast = _client.GetXmlForecastFromLatLon(setupData.Longitude, setupData.Latitude, _daysNumericUpDown.Text);
					result.Explaination = "Success";
				}
				catch (Exception ex)
				{
					result.Success = false;
					result.XmlForecast = string.Empty;
					result.Explaination = ex.Message;
				}

			}
			else
			{
				result.Success = false;
				result.XmlForecast = string.Empty;
				result.Explaination = "Failed to get forecast - NDFD client was not intialized.";
			}

			return result;
		}


		/// <summary>
		/// Contains all the logic for enabling/disabling UI controls to match
		/// the requested application state.
		/// </summary>
		/// <param name="state"></param>
		private void SetState(AppState state)
		{
			_appState = state;

			switch (state)
			{
				case AppState.NeedZip:
					_statusLabel.Text = "Ready for ZIP code";
					_zipTextBox.Enabled = true;
					_zipTextBox.Focus();
					_zipTextBox.SelectAll();
					_forecastButton.Enabled = false;
					_xmlTree.Clear();
					break;
				case AppState.GettingLocation:
					_statusLabel.Text = "Waiting for logitude/latitude";
					_zipTextBox.Enabled = false;
					_forecastButton.Enabled = false;
					break;
				case AppState.ReadyForGetForcast:
					_statusLabel.Text = "Ready to request forecast";
					_zipTextBox.Enabled = true;
					_forecastButton.Enabled = true;
					_forecastButton.Focus();
					break;
				case AppState.GettingForecast:
					_statusLabel.Text = "Waiting for forecast";
					_forecastButton.Enabled = false;
					_xmlTree.Clear();
					break;
				case AppState.HaveForecast:
					_statusLabel.Text = "Forecast successfully received";
					_forecastButton.Enabled = true;
					break;
			}
		}


		/// <summary>
		/// 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void _daysNumericUpDown_ValueChanged(object sender, EventArgs e)
		{
			//if (_appState == AppState.HaveForecast)
			//	SetState(AppState.ReadyForGetForcast);		
		}
	}


	/// <summary>
	/// Used to bundle results from GetLonLatFromZip().
	/// </summary>
	internal class GetLonLatResult
	{
		public bool Success { get; set; }
		public PointF LonLat { get; set; }
		public string Explaination { get; set; }


		public GetLonLatResult() {}

		public GetLonLatResult(bool bSuccess, PointF lonLat, string explaination)
		{
			Success = bSuccess;
			LonLat = lonLat;
			Explaination = explaination;
		}
	}


	internal class ForecastSetupData
	{
		public decimal Longitude { get; set; }
		public decimal Latitude { get; set; }
		public string DurationDays { get; set; }

		public ForecastSetupData()
		{
		}

		public ForecastSetupData(decimal longitude, decimal latitiude, string durationDays)
		{
			Longitude = longitude;
			Latitude = latitiude;
			DurationDays = durationDays;
		}
	}


	internal class GetXmlForecastResult
	{
		public bool Success { get; set; }
		public string XmlForecast { get; set; }
		public string Explaination { get; set; }


		public GetXmlForecastResult() {}

		public GetXmlForecastResult(bool bSuccess,string xmlForecast, string explaination)
		{
			Success = bSuccess;
			XmlForecast = xmlForecast;
			Explaination = explaination;
		}	
	}
}
